<template>
  <ModalDialog
    ref="updateModal"
    is-page
    @close="$router.push({ name: 'admin.user_roles' })"
  >
    <ModalHeader>
      <ModalTitle>{{ $t("app.admin.user_roles.update.title") }}</ModalTitle>
    </ModalHeader>
    <ModalForm
      :is-loading="userRoleFetch.isFetching"
      @submit="submit"
    >
      <template #default>
        <AlertError v-if="form.errors && form.errors?.unknown">
          {{ form.errors.unknown }}
        </AlertError>
        <template v-if="userRoleFetch.hasErrors">
          <ErrorState />
        </template>
        <template v-else>
          <FormInput
            id="name"
            :errors="form.errors.name"
          >
            <FormLabel>{{ $t("app.admin.user_roles.form.name") }}</FormLabel>
            <FormText
              v-model="form.values.name"
              autocomplete="off"
              name="name"
              required
            />
          </FormInput>
          <FormInput
            id="permissions"
            :errors="form.errors.permissions"
          >
            <FormLabel>{{ $t("app.admin.user_roles.form.permissions") }}</FormLabel>
            <UserRolePermissionsSelect
              v-model="form.values.permissions"
              required
            />
          </FormInput>
        </template>
      </template>
      <template #footer>
        <FormSubmit
          :processing="form.isFetching"
          class="btn btn-black"
        >
          {{ $t("app.admin.user_roles.update.submit") }}
        </FormSubmit>
        <UserRoleDeleteModal
          v-if="hasPermission(Permission.ADMIN_USER_ROLES_DELETE)"
          :id="String($route.params.id)"
          v-slot="{ open }"
          @confirm="onDeleted"
        >
          <button
            class="btn btn-outline-red"
            type="button"
            @click="open"
          >
            <ButtonIcon :icon="TrashCanRegularIcon">
              {{ $t("app.admin.user_roles.delete.title") }}
            </ButtonIcon>
          </button>
        </UserRoleDeleteModal>
      </template>
    </ModalForm>
  </ModalDialog>
</template>

<script lang="ts" setup>
import ModalDialog from "@/js/contexts/shared/ui/components/modal/ModalDialog.vue";
import ModalHeader from "@/js/contexts/shared/ui/components/modal/ModalHeader.vue";
import FormInput from "@/js/contexts/form/ui/components/FormInput.vue";
import FormText from "@/js/contexts/form/ui/components/FormText.vue";
import AlertError from "@/js/contexts/shared/ui/components/alert/AlertError.vue";
import UserRolePermissionsSelect from "@/js/contexts/admin/user-roles/ui/components/UserRolePermissionsSelect.vue";
import ModalTitle from "@/js/contexts/shared/ui/components/modal/ModalTitle.vue";
import FormSubmit from "@/js/contexts/form/ui/components/FormSubmit.vue";
import FormLabel from "@/js/contexts/form/ui/components/FormLabel.vue";
import ModalForm from "@/js/contexts/shared/ui/components/modal/ModalForm.vue";
import useModal from "@/js/contexts/shared/composables/useModal";
import useForm from "@/js/contexts/shared/composables/useForm";
import type { UserRoleUpdate } from "@/js/contexts/shared/models/UserRole";
import ErrorState from "@/js/contexts/shared/ui/components/error/ErrorState.vue";
import useFetch from "@/js/contexts/shared/composables/useFetch";
import { Permission } from "@/js/contexts/shared/models/Permission";
import useAuth from "@/js/contexts/auth/composables/useAuth";
import UserRoleDeleteModal from "@/js/contexts/admin/user-roles/ui/modals/UserRoleDeleteModal.vue";
import ButtonIcon from "@/js/contexts/shared/ui/components/button/ButtonIcon.vue";
import TrashCanRegularIcon from "@/svg/icons/trash-can-regular.svg?component";
import useNotification from "@/js/contexts/shared/composables/useNotification";

type Response = { name: string; permissions: Permission[] };

const emit = defineEmits<{
  fetch: [];
}>();

const { t } = useI18n();
const route = useRoute();
const { successNotification } = useNotification();
const { hasPermission } = useAuth();
const updateModal = ref<InstanceType<typeof ModalDialog>>();
const modal = useModal(updateModal);
const userRoleFetch = useFetch(`/api/admin/user-roles/${route.params.id}`, { isFetching: true });
const form = useForm<UserRoleUpdate>({
  name: "",
  permissions: [],
});

onMounted(() => {
  userRoleFetch.get<Response>().then((response) => {
    form.values.name = response.data.name;
    form.values.permissions = response.data.permissions;
    form.updatedValues();
  });
});

const submit = () => {
  form.put(`/api/admin/user-roles/${route.params.id}`, {
    onSuccess: () => {
      emit("fetch");
      modal.close();
      successNotification(t("app.admin.user_roles.update.success"));
    },
  });
};
const onDeleted = () => {
  emit("fetch");
  modal.close();
};
</script>
